<?php

/******************** MOLE -- Copyright 2008 *************************
 *
 * By
 * 		Erik Clairiot <ipingu *AT* clairiot _DOT_ com>
 * 		Remi Melisson <tospecify>
 *
 * Part of Mole.
 * Service that allows users to connect and register on the server.
 *
 * Mole is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mole is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mole; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *********************************************************************/

require_once('Service.php');
require_once('../libs/database/MySQLQuery.php');
require_once('../conf/Params.inc.php');

/**
 * This class is a service that allows a client to connect to a server.
 */
abstract class Connection implements Service {

    protected static $errcode = array(
	'ok' => 200,
	'exists' => 101,
	'invalid_name' => 102,
	'invalid_address' => 103,
	'address_used' => 104);

    protected static $errtext = array(
	'ok' => 'You are successfully connected',
	'exists' => 'Your login is still in the database',
	'invalid_name' => 'Please specify a valid login',
	'invalid_address' => 'Your address (ip & port) is erronated',
	'address_used' => 'Your address ip is used. Please change your port');

};

/**
 * In this case, we store only 3 informations from the client, which are
 * all required at the service invocation : Login, Ip, Port.
 * It is stored in a MySQL database.
 */
class SQLConnection extends Connection {

    /**
     * Check the validity of the login execution.
     * IP & Port must be valid, and not used.
     * Login should not be too long and not exists on the server.
     *
     * @param login Login used by the client
     */
    private function checkValidity($login, $ip, $port){
	$dbHandler = MySQLQuery::getConfDB();
	/* Check at first validity of port & ip */
	if (strlen($ip) > 0 &&
	    strlen($ip) <= 15 &&
	    substr_count($ip, '.') == 3 &&
	    $port > 0){

		/* If the same ip is used, we compare both ports. */
		$result = $dbHandler->executeResultSet(
		    'SELECT port FROM '.TABLE_USERS.' WHERE
		    ip="'.$ip.'"');
		$same = 0;
		while ($portToCmp = mysql_fetch_object($result)){
		    if ($port == $portToCmp)
			return 'address_used';
		}

		/* Then validity of login */
		if (strlen($login) > 0 &&
		    strlen($login) < LOGIN_MAX_SIZE){

			/* Login existence check */
			$result = $dbHandler->executeResultSet(
			    'SELECT molid FROM '.TABLE_USERS.' WHERE
			    name="'.$login.'"');
			if ($dbHandler->getNumRows($result) == 0){
			    return 'ok';
			}
			return 'exists';
		    }
		return 'invalid_name';
	    }
	return 'invalid_address';
    }

    /**
     * Executes the service.
     *
     * @param login RPCString : Login used by the client
     * @param ip RPCString : Ip used by the client
     * @param port RPCInteger : Port of the channel command used by the client
     * @return A RPC response
     */
    public function execute($args){
	/* Retrieves datas contained in xmlrpc request.*/
	//return new xmlrpcresp(new xmlrpcval("yes", 'string'));
	echo $args->serialize();
	$login = $args->getParam(0)->scalarVal();
	$ip = $args->getParam(1)->scalarVal();
	$port = $args->getParam(2)->scalarVal();

	$status = SQLConnection::checkValidity($login, $ip, $port);

	/* Valid, we update the database */
	if ($status == 'ok'){
	    $dbHandler = MySQLQuery::getConfDB();
	    $molid = md5($login.time());
	    $dbHandler->execute(
		'INSERT INTO '.TABLE_USERS.' SET name="'.$login.'",
		ip="'.$ip.'", port="'.$port.'", molid="'.$molid.'",
		time="'.time().'"');
	    return new xmlrpcresp(new xmlrpcval($molid, 'string'));
	}

	return new xmlrpcresp(0, self::$errcode[$status], self::$errtext[$status]);
    }
}

?>
