<?php

require_once('Service.php');
require_once('../libs/database/MySQLQuery.php');
require_once('../conf/Params.inc.php');
require_once('../helper/Policy.php');
require_once('../helper/ServicesQueryHelper.php');

abstract class JoinChannel implements Service {

    protected static $errcode = array(
	'created' => 201,
	'joined' => 202,
	'invalid_name' => 101,
	'creation_error' => 102,
	'created_but_not_joined' => 103,
	'join_error' => 104,
	'not_connected' => 110);

    protected static $errtext = array(
	'created' => 'The channel was created, and you joined it.',
	'joined' => 'You joined the channel.',
	'invalid_name' => 'Channel name is invalid.',
	'creation_error' => 'Channel cannot be created.',
	'created_but_not_joined' => 'Channel created, but not joined.',
	'join_error' => 'Join channel failed.',
	'not_connected' => 'You have to be connected to the Mole to do this');

};

class SQLJoinChannel extends JoinChannel {

    public function execute($args){
	/* This service requires that client was previously auth on
	 * the server. Policy class do it for us. */
	if (Policy::checkAuth($args->getParam(0)->scalarVal())){
		$id_user = Policy::getIdByMolid($args->getParam(0)->scalarVal());
		if ($id_user < 0)
		return Policy::getXmlrpcresp();
	} else {
	    return Policy::getXmlrpcresp();
	}
	
	/* Retrieves datas contained in xmlrpc request.*/
	$channel = $args->getParam(1)->scalarVal();
	$status = 'invalid_name';
	if (SQLServicesQueryHelper::channelExists($channel)){
	    $status = SQLServicesQueryHelper::joinChannel($channel, $id_user) ?
		'joined' : 'join_error';
	} else {
	    /* If the chan does not exist, we create it */
	    if (SQLServicesQueryHelper::createChannel($channel)){
		$status = SQLServicesQueryHelper::joinChannel($channel, $id_user) ?
		    'created' : 'created_but_not_joined';
	    } else {
		$status = 'creation_error';
	    }
	}


	/* Cases of success */
	if ($status == 'created' || $status == 'joined'){
	    return new xmlrpcresp(new xmlrpcval(self::$errcode[$status], 'int'));
	}
	
	return new xmlrpcresp(0, self::$errcode[$status],
	    self::$errtext[$status]);
    }
}

?>
